var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const pull = require('pull-stream');
const nodejs = require('nodejs-mobile-react-native');
const ssbKeys = require('react-native-ssb-client-keys');
const rnChannelPlugin = require('multiserver-rn-channel');
const noAuthPlugin = require('multiserver/plugins/noauth');
const MultiServer = require('multiserver');
const muxrpc = require('muxrpc');
function toSodiumKeys(keys) {
    if (!keys || !keys.public)
        return null;
    return {
        publicKey: Buffer.from(keys.public.replace('.ed25519', ''), 'base64'),
        secretKey: Buffer.from(keys.private.replace('.ed25519', ''), 'base64'),
    };
}
function objMapDeep(origin, transform) {
    return Object.keys(origin).reduce((acc, key) => {
        if (typeof origin[key] === 'object') {
            acc[key] = objMapDeep(origin[key], transform);
        }
        else {
            acc[key] = transform(origin[key]);
        }
        return acc;
    }, {});
}
function pipe(first, ...cbs) {
    let res = first;
    for (let i = 0, n = cbs.length; i < n; i++)
        res = cbs[i](res);
    return res;
}
function syncToAsync(str) {
    return str === 'sync' ? 'async' : str;
}
function applyPlugins(client, plugins) {
    for (const plugin of plugins) {
        client[plugin.name] = plugin.init(client);
    }
    return client;
}
function hackId(client, keys) {
    client.id = keys.id;
    return client;
}
function loadKeys(path) {
    return new Promise((resolve, reject) => {
        ssbKeys.load(path, (err, val) => {
            if (err)
                reject(err);
            else
                resolve(val);
        });
    }).catch(() => loadKeys(path));
}
export default function ssbClient(k, manifest) {
    const sanitizedManifest = objMapDeep(manifest, syncToAsync);
    const plugins = [];
    function builder(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            const keys = typeof k === 'string' ? yield loadKeys(k) : k;
            const ms = MultiServer([
                [
                    rnChannelPlugin(nodejs.channel),
                    noAuthPlugin({ keys: toSodiumKeys(keys) }),
                ],
            ]);
            const address = 'channel~noauth:' + keys.public.replace('.ed25519', '');
            ms.client(address, (err, stream) => {
                if (err) {
                    cb(err);
                }
                else {
                    const client = pipe(muxrpc(sanitizedManifest, null)(), c => hackId(c, keys), c => applyPlugins(c, plugins));
                    pull(stream, client.createStream(), stream);
                    cb(null, client);
                }
            });
        });
    }
    builder.use = function use(plugin) {
        plugins.push(plugin);
        return builder;
    };
    builder.callPromise = function callPromise() {
        return new Promise((resolve, reject) => {
            builder((err, val) => {
                if (err)
                    reject(err);
                else
                    resolve(val);
            });
        });
    };
    return builder;
}
//# sourceMappingURL=index.js.map